
/**************************************************************************
**  (c) Copyright 2002, Andromeda Technology & Automation
** This is free software; you can redistribute it and/or modify it under the
** terms of the GNU General Public License, see the file COPYING.
***************************************************************************
** MODULE INFORMATION *
***********************
**      FILE NAME      : message.h
**      SYSTEM NAME    : 
**      VERSION NUMBER : $Revision: 1.3 $
**
**  DESCRIPTION      :  Classes to for handling client messages
**
**  EXPORTED OBJECTS : 
**  LOCAL    OBJECTS : 
**  MODULES  USED    :
***************************************************************************
**  ADMINISTRATIVE INFORMATION *
********************************
**      ORIGINAL AUTHOR : Arjen Baart - arjen@andromeda.nl
**      CREATION DATE   : Sep 16, 2002
**      LAST UPDATE     : Nov 04, 2002
**      MODIFICATIONS   : 
**************************************************************************/

/*****************************
   $Log: message.h,v $
   Revision 1.3  2002/11/09 08:04:27  arjen
   Added a reference to the GPL

   Revision 1.2  2002/11/04 10:13:36  arjen
   Use proper namespace for iostream classes

   Revision 1.1  2002/10/05 10:25:49  arjen
   Creation of gcm_input and a first approach to a web interface

*****************************/

/* static const char *RCSID = "$Id: message.h,v 1.3 2002/11/09 08:04:27 arjen Exp $"; */

#include <iostream>
#include <list>
#include <AXE/String.h>
#include <AXE/date.h>

#include "database.h"

/*
///////////////////////////////////////////////////////////////////////////
//  NAME           : message_buffer
//  BASECLASS      : 
//  MEMBERS        :
//  OPERATORS      :
//  METHODS        : rewind()
//
//  DESCRIPTION    : 
//
//  RELATIONS      :
//  SEE ALSO       :
//  LAST MODIFIED  : Nov 04, 2002
///////////////////////////////////////////////////////////////////////////
*/

class message_buffer
{
   std::istream       *input;
   std::list<String>  buffer;

   std::list<String>::iterator   next_line;

public:

   message_buffer()
   {
      input = 0;
      next_line = buffer.begin();
   }

   message_buffer(std::istream *in)
   {
      input = in;
      next_line = buffer.begin();
   }

   void from(std::istream *in)
   {
      input = in;
   }

   friend bool operator >> (message_buffer &, String &);

   void rewind()
   {
      next_line = buffer.begin();
   }

   void operator ++()
   {
      if (next_line != buffer.end())
      {
         next_line++;
      }
   }

   void operator --()
   {
      if (next_line != buffer.begin())
      {
         next_line--;
      }
   }
};

/*
///////////////////////////////////////////////////////////////////////////
//  NAME           : client_message
//  BASECLASS      : 
//  MEMBERS        :
//  OPERATORS      :
//  METHODS        : classify()
//                   enter()
//
//  DESCRIPTION    : 
//
//  RELATIONS      :
//  SEE ALSO       :
//  LAST MODIFIED  : Nov 04, 2002
///////////////////////////////////////////////////////////////////////////
*/

class client_message
{
   String     hostname;      //  Where the message came from (FQDN)
   UTC        arrival;       //  When we got the message.
   String     service;       //  Service that created the message

   bool       mail_header;   //  Does the message contain a mail header ?
   bool       gpg_encrypted; //  Is the message encrypted ?

   double     certainty;     //  How certain are we about the message
   enum
   {
      UNKNOWN, SYSLOG, ACCESSLOG, ERRORLOG
   }  classification;


   message_buffer    input;
   gnucomo_database  database;

public:

   client_message(std::istream *in, gnucomo_database db);

   double classify(String host, UTC arrival = Now(), String serv = "");
   int    enter();
};

